from papermage.recipes.core_recipe import CoreRecipe
from papermage.recipes.minimal_pdf_recipe import (
    MinimalTextImageRecipe,
    MinimalTextOnlyRecipe,
)
from papermage.recipes.text_recipe import TextRecipe

__all__ = ["CoreRecipe", "TextRecipe", "MinimalTextImageRecipe", "MinimalTextOnlyRecipe"]



"""

@kylel

"""

import json
import os
import pathlib

from papermage.magelib import Document
from papermage.recipes import CoreRecipe
from papermage.visualizers.visualizer import plot_entities_on_page

# load doc
recipe = CoreRecipe()
# pdfpath = pathlib.Path(__file__).parent.parent / "tests/fixtures/1903.10676.pdf"
# pdfpath = pathlib.Path(__file__).parent.parent / "tests/fixtures/2304.02623v1.pdf"
pdfpath = pathlib.Path(__file__).parent.parent / "tests/fixtures/2020.acl-main.447.pdf"
# pdfpath = pathlib.Path(__file__).parent.parent / "tests/fixtures/2305.14772.pdf"
doc = recipe.from_pdf(pdf=pdfpath)

# visualize tokens
page_id = 0
plot_entities_on_page(page_image=doc.images[page_id], entities=doc.pages[page_id].tokens)

# visualize tables
page_id = 5
tables = doc.pages[page_id].intersect_by_box("tables")
plot_entities_on_page(page_image=doc.images[page_id], entities=tables)
for table in tables:
    print(table.text)

# visualize figures
figures = doc.pages[page_id].intersect_by_box("figures")
for figure in figures:
    print(figure.text)
plot_entities_on_page(page_image=doc.images[page_id], entities=figures)

# visualize blocks
blocks = doc.pages[page_id].intersect_by_box("blocks")
for block in blocks:
    print(block.text)
plot_entities_on_page(page_image=doc.images[page_id], entities=blocks)

# visualize sections
page_id = 4
sections = doc.pages[page_id].intersect_by_box("sections")
for section in sections:
    print(section.text)
plot_entities_on_page(page_image=doc.images[page_id], entities=sections)


# get the image of a page and its dimensions
page_image = doc.images[page_id]
page_w, page_h = page_image.pilimage.size

# get the bounding box of a figure
figure_box = figures[0].boxes[0]

# convert it
figure_box_xy = figure_box.to_absolute(page_width=page_w, page_height=page_h).xy_coordinates

# crop the image using PIL
page_image._pilimage.crop(figure_box_xy)


##### BUG

# weird.  this says no figures on page 8, but there are
doc.pages[8].intersect_by_box("figures")
page = doc.pages[8]

# get figures on 8th page.
figures = [figure for figure in doc.figures if figure.boxes[0].page == 8]
assert len(figures) > 0
print(f"{figures[0].boxes}")

# weird.. it's on 8th page
print(f"{figure.boxes[0].xy_coordinates}")

plot_entities_on_page(page_image=doc.images[8], entities=figures)


# but yet it doesnt intersect pages
figure.intersect_by_box("pages")

print(f"{page.boxes[0].xy_coordinates}")

figure.boxes[0].is_overlap(page.boxes[0])



"""

Minimal recipe that handles PDF binaries.

@kylel

"""

import logging
from pathlib import Path
from typing import Dict, List, Union

from papermage.magelib import Document, Entity
from papermage.parsers.pdfplumber_parser import PDFPlumberParser
from papermage.rasterizers.rasterizer import PDF2ImageRasterizer
from papermage.recipes.recipe import Recipe
from papermage.utils.annotate import group_by


class MinimalTextImageRecipe(Recipe):
    def __init__(
        self,
        dpi: int = 300,
    ):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.dpi = dpi
        self.logger.info("Instantiating recipe...")
        self.parser = PDFPlumberParser()
        self.rasterizer = PDF2ImageRasterizer()
        self.logger.info("Finished instantiating recipe")

    def from_pdf(self, pdf: Path) -> Document:
        self.logger.info("Parsing document...")
        doc = self.parser.parse(input_pdf_path=pdf)

        self.logger.info("Rasterizing document...")
        images = self.rasterizer.rasterize(input_pdf_path=pdf, dpi=self.dpi)
        doc.annotate_images(images=list(images))
        self.rasterizer.attach_images(images=images, doc=doc)
        return doc


class MinimalTextOnlyRecipe(Recipe):
    def __init__(self):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.logger.info("Instantiating recipe...")
        self.parser = PDFPlumberParser()
        self.logger.info("Finished instantiating recipe")

    def from_pdf(self, pdf: Path) -> Document:
        self.logger.info("Parsing document...")
        doc = self.parser.parse(input_pdf_path=pdf)

        self.logger.info("Rasterizing document...")
        return doc


if __name__ == "__main__":
    import argparse
    import json

    parser = argparse.ArgumentParser()
    parser.add_argument("--pdf", required=True, type=str, help="Path to PDF file.")
    parser.add_argument("--output", type=str, help="Path to output JSON file.")
    parser.add_argument("--recipe", type=str, default="text_only", help="Recipe to use.")
    parser.add_argument("--dpi", type=int, default=300, help="DPI for rasterization.")
    args = parser.parse_args()

    if args.recipe == "text_image":
        recipe = MinimalTextImageRecipe(dpi=args.dpi)
        doc = recipe.from_pdf(pdf=args.pdf)
        with open(args.output, "w") as f:
            json.dump(doc.to_json(), f, indent=2)
    elif args.recipe == "text_only":
        recipe = MinimalTextOnlyRecipe()
        doc = recipe.from_pdf(pdf=args.pdf)
        with open(args.output, "w") as f:
            json.dump(doc.to_json(), f, indent=2)
    else:
        raise ValueError(f"Invalid recipe: {args.recipe}")



"""

@kylel

"""

import pathlib
import unittest

from papermage.recipes import MinimalTextImageRecipe, MinimalTextOnlyRecipe


class TestMinimalTextOnlyRecipe(unittest.TestCase):
    def setUp(self):
        self.fixture_path = pathlib.Path(__file__).parent.parent / "fixtures"
        self.recipe = MinimalTextOnlyRecipe()

    def test_stability(self):
        doc = self.recipe.run(self.fixture_path / "1903.10676.pdf")
        self.assertTrue(doc.symbols.startswith("Field\nTask\nDataset\nSOTA\nB ERT -Base\nS CI B ERT"))
        self.assertListEqual([t.text for t in doc.tokens[:5]], ["Field", "Task", "Dataset", "SOTA", "B"])


class TestMinimalTextImageRecipe(unittest.TestCase):
    def setUp(self):
        self.fixture_path = pathlib.Path(__file__).parent.parent / "fixtures"
        self.recipe_high_quality = MinimalTextImageRecipe(dpi=300)
        self.recipe_low_quality = MinimalTextImageRecipe(dpi=72)

    def test_high_quality(self):
        doc = self.recipe_high_quality.run(self.fixture_path / "1903.10676.pdf")
        self.assertTrue(doc.symbols.startswith("Field\nTask\nDataset\nSOTA\nB ERT -Base\nS CI B ERT"))
        self.assertListEqual([t.text for t in doc.tokens[:5]], ["Field", "Task", "Dataset", "SOTA", "B"])
        self.assertEqual(len(doc.images), 1)
        self.assertEqual(doc.images[0].pilimage.height, 3509)
        self.assertEqual(doc.images[0].pilimage.width, 2480)

    def test_low_quality(self):
        doc = self.recipe_low_quality.run(self.fixture_path / "1903.10676.pdf")
        self.assertTrue(doc.symbols.startswith("Field\nTask\nDataset\nSOTA\nB ERT -Base\nS CI B ERT"))
        self.assertListEqual([t.text for t in doc.tokens[:5]], ["Field", "Task", "Dataset", "SOTA", "B"])
        self.assertEqual(len(doc.images), 1)
        self.assertEqual(doc.images[0].pilimage.height, 842)
        self.assertEqual(doc.images[0].pilimage.width, 595)
